/*-- $Id$ -- 

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================
 
    Copyright (C) 1999 The Apache Software Foundation. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:
 
 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.
 
 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
 
 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.
 
 4. The names "FOP" and  "Apache Software Foundation"  must not be used to
    endorse  or promote  products derived  from this  software without  prior
    written permission. For written permission, please contact
    apache@apache.org.
 
 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.
 
 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 James Tauber <jtauber@jtauber.com>. For more  information on the Apache 
 Software Foundation, please see <http://www.apache.org/>.
 
 */

package org.apache.fop.dom.svg;

import org.w3c.dom.svg.*;
import org.w3c.dom.*;
import org.w3c.dom.events.*;

/**
 * class representing a line in an SVG Area
 */
public class SVGLineElementImpl extends GraphicElement implements SVGLineElement {
	/** x-coordinate of start */
	protected SVGAnimatedLength x1;

	/** y-coordinate of start */
	protected SVGAnimatedLength y1;

	/** x-coordinate of end */
	protected SVGAnimatedLength x2;

	/** y-coordinate of end */
	protected SVGAnimatedLength y2;

	/**
	 * construct a line graphic
	 *
	 * @param x1 x-coordinate of start
	 * @param y1 y-coordinate of start
	 * @param x2 x-coordinate of end
	 * @param y2 y-coordinate of end
	 */
	public SVGLineElementImpl()
	{
	}

	public SVGRect getBBox()
	{
		SVGRect rect = new SVGRectImpl();
		rect.setX(Math.min(x1.getBaseVal().getValue(), x2.getBaseVal().getValue()));
		rect.setY(Math.min(y1.getBaseVal().getValue(), y2.getBaseVal().getValue()));
		rect.setWidth(Math.abs(x1.getBaseVal().getValue() - x2.getBaseVal().getValue()));
		rect.setHeight(Math.abs(y1.getBaseVal().getValue() - y2.getBaseVal().getValue()));
		return rect;
	}

  public SVGAnimatedLength getX1( )
  {
  	return x1;
  }

  public void setX1( SVGAnimatedLength x1 )
  {
	this.x1 = x1;
  }

  public SVGAnimatedLength getY1( )
  {
  	return y1;
  }

  public void setY1( SVGAnimatedLength y1 )
  {
	this.y1 = y1;
  }

  public SVGAnimatedLength getX2( )
  {
  	return x2;
  }

  public void setX2( SVGAnimatedLength x2 )
  {
	this.x2 = x2;
  }

  public SVGAnimatedLength getY2( )
  {
  	return y2;
  }

  public void setY2( SVGAnimatedLength y2 )
  {
	this.y2 = y2;
  }
}
