/*-- $Id$ -- 

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================
 
    Copyright (C) 1999 The Apache Software Foundation. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:
 
 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.
 
 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
 
 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.
 
 4. The names "FOP" and  "Apache Software Foundation"  must not be used to
    endorse  or promote  products derived  from this  software without  prior
    written permission. For written permission, please contact
    apache@apache.org.
 
 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.
 
 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 James Tauber <jtauber@jtauber.com>. For more  information on the Apache 
 Software Foundation, please see <http://www.apache.org/>.
 
 */
package org.apache.fop.fo.expr;

import org.apache.fop.datatypes.*;
import org.apache.fop.fo.Property;
import org.apache.fop.fo.LengthProperty;
import org.apache.fop.fo.FObj;
import org.apache.fop.fo.flow.ListItem;

public class LabelEndFunction extends FunctionBase {

    public int nbArgs() { return 0; }

    public Property eval(Property[] args, PropertyInfo pInfo)
	throws PropertyException {

	Length distance
	    = pInfo.getPropertyList()
	    .get("provisional-distance-between-starts").getLength();
	Length separation
	    = pInfo.getPropertyList()
	    .getNearestSpecified("provisional-label-separation").getLength();

	FObj item = pInfo.getFO();
	while(item != null && !(item instanceof ListItem)) {
	    item = item.getParent();
	}
	if(item == null) {
	    throw new PropertyException("label-end() called from outside an fo:list-item");
	}
	Length startIndent = item.properties.get("start-indent").getLength();
	
	LinearCombinationLength labelEnd = new LinearCombinationLength();

	// Should be CONTAINING_REFAREA but that doesn't work
	LengthBase base = new LengthBase(item, pInfo.getPropertyList(),
					 LengthBase.CONTAINING_BOX);
	PercentLength refWidth = new PercentLength(1.0, base);

	labelEnd.addTerm(1.0, refWidth);
	labelEnd.addTerm(-1.0, distance);
	labelEnd.addTerm(-1.0, startIndent);
	labelEnd.addTerm(1.0, separation);

	return new LengthProperty(labelEnd);
    }
}
