/*-- $Id$ -- 

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================
 
    Copyright (C) 1999 The Apache Software Foundation. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:
 
 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.
 
 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
 
 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.
 
 4. The names "FOP" and  "Apache Software Foundation"  must not be used to
    endorse  or promote  products derived  from this  software without  prior
    written permission. For written permission, please contact
    apache@apache.org.
 
 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.
 
 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 James Tauber <jtauber@jtauber.com>. For more  information on the Apache 
 Software Foundation, please see <http://www.apache.org/>.
 
 */

package org.apache.fop.svg;

// FOP
import org.apache.fop.fo.*;
import org.apache.fop.layout.Area;
import org.apache.fop.layout.FontState;
import org.apache.fop.apps.FOPException;

import org.apache.fop.dom.svg.*;
import org.apache.fop.dom.svg.SVGArea;

import org.w3c.dom.svg.*;

import java.util.StringTokenizer;

/**
 *
 */
public class Symbol extends SVGObj {

	/**
	 * inner class for making Symbol objects.
	 */
	public static class Maker extends FObj.Maker {

		/**
		 * make a Symbol object.
		 *
		 * @param parent the parent formatting object
		 * @param propertyList the explicit properties of this object
		 *
		 * @return the Symbol object
		 */
		public FObj make(FObj parent, PropertyList propertyList) throws FOPException
		{
			return new Symbol(parent, propertyList);
		}
	}

	/**
	 * returns the maker for this object.
	 *
	 * @return the maker for Symbol objects
	 */
	public static FObj.Maker maker() {
		return new Symbol.Maker();
	}

	/**
	 * constructs a Symbol object (called by Maker).
	 *
	 * @param parent the parent formatting object
	 * @param propertyList the explicit properties of this object
	 */
	protected Symbol(FObj parent, PropertyList propertyList) {
		super(parent, propertyList);
		this.name = "svg:symbol";
	}

	public SVGElement createGraphic()
	{
		String box = this.properties.get("viewBox").getString();
		StringTokenizer st = new StringTokenizer(box, " ");
		float x = 0;
		float y = 0;
		float width = 0;
		float height = 0;
		try {
			if(st.hasMoreTokens()) {
				x = Double.valueOf(st.nextToken()).floatValue();
			}
			if(st.hasMoreTokens()) {
				y = Double.valueOf(st.nextToken()).floatValue();
			}
			if(st.hasMoreTokens()) {
				width = Double.valueOf(st.nextToken()).floatValue();
			}
			if(st.hasMoreTokens()) {
				height = Double.valueOf(st.nextToken()).floatValue();
			}
		} catch(Exception e) {
		}
		SVGRect rect = new SVGRectImpl();
		rect.setX(x);
		rect.setY(y);
		rect.setWidth(width);
		rect.setHeight(height);
		SVGSymbolElementImpl graphic;
		graphic = new SVGSymbolElementImpl();
		String id = this.properties.get("id").getString();
		graphic.setId(id);
		graphic.setViewBox(new SVGAnimatedRectImpl(rect));

		int numChildren = this.children.size();
		for (int i = 0; i < numChildren; i++) {
			FONode child = (FONode) children.elementAt(i);
			if(child instanceof GraphicsCreator) {
				SVGElement impl = ((GraphicsCreator)child).createGraphic();
				graphic.appendChild(impl);
			}
		}

		return graphic;
	}
}
